#include "StdAfx.h"
#include "Convertor.h"

namespace
{

//=====================================================================================//
//                            void writeToStreamCallback()                             //
//=====================================================================================//
void CALLBACK writeToStreamCallback(LPVOID lpParam,LPCVOID lpData, DWORD nLen)
{
	std::ostream& stream = *reinterpret_cast<std::ostream*>(lpParam);
	stream.write(reinterpret_cast<const char*>(lpData),nLen);
	stream.flush();
}

}

//=====================================================================================//
//                                   class Convertor                                   //
//=====================================================================================//
Convertor::Convertor(const std::string& file_name)
: m_grabber(file_name,setupGrabberOptions())
, m_mediaDetector(file_name)
, m_observer(0)
{
}

Convertor::~Convertor()
{
}
//	convert open avi file to flv
bool Convertor::convertToFlv(const std::string& flv_file_name)
{
	std::ofstream stream(flv_file_name.c_str(),std::ios::binary);

	if(stream)
	{
		FTVIDEOINFO ftvi = {0};
		FTAUDIOINFO ftai = {0};
		FTFLVSTREAMINFO ftflvsi = {0,0,&writeToStreamCallback,&stream};

		setupFlashToolStreamInfo(ftflvsi,ftvi,ftai);

		HFTFLVSTREAM hFlvStream = ftFlvStreamCreate(&ftflvsi);
		if(hFlvStream)
		{
			unsigned int frame_rate = 15;
			if(m_mediaDetector.getVideoDataFormat())
				frame_rate = m_mediaDetector.getVideoDataFormat()->getFrameRate();

			FlvDataCollector collector(hFlvStream,this,ftvi.dwWidth,ftvi.dwHeight,frame_rate);
			m_grabber.run(&collector);
			collector.flush();
			ftFlvStreamDestroy(hFlvStream);

			return true;
		}
	}

	return false;
}
//	set observer
void Convertor::setObserver(ConvertorObserver* observer)
{
	m_observer = observer;
}
//	setup FlashTool structure
void Convertor::setupFlashToolStreamInfo(FTFLVSTREAMINFO& ftflvsi,FTVIDEOINFO& ftvi,FTAUDIOINFO& ftai)
{
	const MDVideoDataFormat* video_data_format = m_mediaDetector.getVideoDataFormat();
	const MDAudioDataFormat* audio_data_format = m_mediaDetector.getAudioDataFormat();

	if(video_data_format)
	{
		ftvi.dwCodec = FT_VIDEOCODEC_SCREENVIDEO;
		ftvi.dwWidth = video_data_format->getWidth();
		ftvi.dwHeight = video_data_format->getHeight();

		ftflvsi.lpVideoInfo = &ftvi;
	}

	if(audio_data_format)
	{
		ftai.dwFormat = FT_SOUNDFORMAT_UNCOMPRESSED;

		//	setup sound rate
		const unsigned int samples = audio_data_format->getSamplesPerSec();
		if(samples <= 5512) ftai.dwRate = FT_SOUNDRATE_5KHZ;
		else if((samples > 5512) && (samples <= 11025)) ftai.dwRate = FT_SOUNDRATE_11KHZ;
		else if((samples > 11025) && (samples <= 22050)) ftai.dwRate = FT_SOUNDRATE_22KHZ;
		else if(((samples > 22050) && (samples <= 44100)) || (samples > 44100)) ftai.dwRate = FT_SOUNDRATE_44KHZ;

		//	setup sound size
		const unsigned int bits = audio_data_format->getBitsPerSample();
		if(bits == 8) ftai.dwSize = FT_SOUNDSIZE_8BIT;
		else ftai.dwSize = FT_SOUNDSIZE_16BIT;

		//	setup sound type
		const unsigned int channels = audio_data_format->getChannels();
		if(channels == 1) ftai.dwType = FT_SOUNDTYPE_MONO;
		else ftai.dwType = FT_SOUNDTYPE_STEREO;

		ftflvsi.lpAudioInfo = &ftai;
	}
}
//	process progress of converting
void Convertor::onProgress(unsigned int timestamp)
{
	if(m_observer) 
		m_observer->onProgress(timestamp);
}
//	setup grabber options
unsigned int Convertor::setupGrabberOptions()
{
	unsigned int result = 0;

	if(m_mediaDetector.getVideoDataFormat()) 
		result |= Grabber::Options::Video;
	if(m_mediaDetector.getAudioDataFormat()) 
		result |= Grabber::Options::Audio;

	return result;
}
